local ADDON_NAME, ns = ...

local OID = ns.OID
local lib = ns.lib
local L = ns.L

-- dropdown menu
local lmenu = CreateFrame('Frame', 'OakInterruptsDisplay_Launcher', UIParent, 'UIDropDownMenuTemplate')

local SubsectionDivider = {
    text = "",
    isTitle = true,
    notCheckable = true,
    notClickable = true,
    iconOnly = true,
    icon = "Interface\\Common\\UI-TooltipDivider-Transparent",
    iconInfo = {
        tCoordLeft = 0,
        tCorrdRight = 1,
        tCoordTop = 0,
        tCoordBottom = 1,
        tSizeX = 0,
        tFitDropDownSizeX = true,
        tSizeY = 0
    }
}

local Cancel = {
    text = L["Cancel"],
    notCheckable = true,
}

local function BuildHeader(heading)
    return {
        text = heading,
        isTitle = 1,
        notCheckable = true
    }
end

local header = setmetatable({}, { __index = function(t, k)
        local v = BuildHeader(k)
        rawset(t, k, v)
        return v
    end })

local function BuildMenu()
    local menuinfo = {}
    for k, v in pairs(ns.displays) do
        if not menuinfo[1] then menuinfo[1] = header['Active displays'] end
        table.insert(menuinfo, {
            text = k,
            arg1 = v,
            checked = function(button) return not button.arg1.inactive end,
            keepShownOnClick = 1,
            func = function(self, arg1, arg2, checked)
                    if checked then
                        ns.display.Show(arg1)
                    else
                        ns.display.Hide(arg1)
                    end
                end
        })
    end
    tinsert(menuinfo, SubsectionDivider)
    tinsert(menuinfo, header[L['Other']])
    -- hook for minimap icon
    if OID.Minimap and OID.Minimap.MenuHide then
        tinsert(menuinfo, OID.Minimap.MenuHide)
    end
    tinsert(menuinfo, Cancel)

    EasyMenu(menuinfo, lmenu, 'cursor', 0, 0, nil, 1)
end

-- launcher
local function OnClick(self, button)
    if button == 'RightButton' then
        BuildMenu()
    else
        ns.display.ToggleVisibility()
    end
end

local launcher = {
    type = 'launcher',
    icon = 'Interface\\Icons\\Ability_Kick',
    OnClick = OnClick
}

launcher.OnTooltipShow = function(tooltip)
    if not tooltip or not tooltip.AddLine then return end
    tooltip:AddLine(L['Oak Interrupts Display'])
    tooltip:AddLine(lib.colourAction(L["<Left Click> to toggle active displays"]), 1, 1, 1)
    tooltip:AddLine(lib.colourAction(L["<Right Click> to open menu"]), 1, 1, 1)
end

LibStub('LibDataBroker-1.1'):NewDataObject(L['Oak Interrupts Display'], launcher)
OID.launcher = launcher
