local addon_name, ns = ...

local Printer = LibStub:GetLibrary("OakPrint-1.0")(addon_name .. "_FramePool")

local FramePool = { Printer = Printer }

--------------------------------------------------------------------------------
-- FramePool

-- We use closures to hide the implementation (pool) from the caller so
-- methods are defined within the NewPool method.

function FramePool:NewPool(name, parent, type, cleanup)
    local pool = {}
    local framepool = { name = name }

    local count = 0
    function framepool:NewFrame()
        local f = tremove(pool)
        if f then
            Printer.debug(3, "Reusing frame %s", f:GetName() or tostring(f))
            return f, false
        else
            count = count + 1
            local frame_name = name and name .. count
            f = CreateFrame(type or "Frame", frame_name, parent or UIParent)
            Printer.debug(3, "Creating new frame %s", frame_name or tostring(f))
            return f, true
        end
    end

    function framepool:Recycle(frame)
        Printer.debug(3, "Recycling %s", frame:GetName() or tostring(frame))
        tinsert(pool, frame)
        frame:Hide()
        if cleanup then
            cleanup(frame)
        end
    end

    function framepool:NewRecyclingPlant(new_cleanup)
        cleanup = new_cleanup
    end
    return framepool
end

ns.FramePool = FramePool
