local OakPrint = LibStub:NewLibrary("OakPrint-1.0", 0)
if not OakPrint then
    return
end

local colour = setmetatable({ "|cFFFF8800", "|cFFFFFF00" } , { __index = function() return "|cFF88FF00" end })

local function BuildString(...)
    if select("#", ...) == 1 then
        return tostring(...)
    end
    return string.format(...)
end

local function BuildMessage(addon_name, level, ...)
    local c = level and colour[level] or "|cff33ff99"
    local prefix = c .. addon_name .. " |||r "
    return prefix .. BuildString(...)
end

function OakPrint:NewPrinter(addon_name)
    local Printer = { debug_level = 0 }

    function Printer.print(...)
        ChatFrame1:AddMessage(BuildMessage(addon_name, nil, ...))
    end

    function Printer.debug(level, ...) 
        if Printer.debug_level >= level then
            ChatFrame1:AddMessage(BuildMessage(addon_name, level, ...))
        end
    end

    function Printer.SetDebug(l)
        l = tonumber(l)
        if l then
            Printer.debug_level = l
            Printer.print("Debug level changed to %d", l)
        else
            Printer.print("Current debug level: %d", Printer.debug_level)
        end
    end

    return Printer
end

setmetatable(OakPrint, { __call = OakPrint.NewPrinter })
