local OakSlash = LibStub:NewLibrary("OakSlash-1.0", 0)
if not OakSlash then
    return
end

local OakPrint = LibStub:GetLibrary("OakPrint-1.0", true)
--------------------------------------------------------------------------------

local function explode(str, val_pattern)
    local t = {}
    for i in string.gmatch(str, val_pattern) do
        table.insert(t, i)
    end
    return t
end

local function BuildSlashHandler(Printer, func_arg)
    local t = type(func_arg)
    if t == "function" then
        return function(msg, editbox)
            local args = explode(msg, "[^%s]+")
            func_arg(unpack(args))
        end
    elseif t == "table" then
        return function(msg, editbox)
            local args = explode(msg, "[^%s]+")
            local functab_level = func_arg
            local l
            for i = 1, #args do
                local v = args[i]
                if not functab_level[v] then
                    Printer.print("Unknown option: %s", v)
                    return
                end
                functab_level = functab_level[v]
                l = i
                if type(functab_level) == "function" then break end
            end
            if type(functab_level) == "function" then
                functab_level(select(l + 1, unpack(args)))
            else
                Printer.print("Error with slash command, must need more arguments")
            end
        end
    end
end

function OakSlash:NewCommand(command, functab, ...)
    if not self.ident then return end
    local Printer = OakPrint(self.ident .. " - command '" .. command .. "'")
    local ident = self.ident .. "_" .. command
    _G['SLASH_' .. ident .. 1] = '/' .. command
    SlashCmdList[ident] = BuildSlashHandler(Printer, functab)
    -- aliases
    for i = 1, select("#", ...) do
        local alias = select(i, ...)
        _G['SLASH_' .. ident .. (i + 1) ] = '/' .. alias
    end
end

function OakSlash:NewCommandGroup(ident)
    assert(type(ident) == "string")
    local t = { ident = ident }
    setmetatable(t, { __index = OakSlash })
    return t
end

setmetatable(OakSlash, { __call = OakSlash.NewCommandGroup })

--[[
-- Embedding

OakSlash.mixinTargets = OakSlash.mixinTargets or {}
local mixins = { "NewCommand" }

local function OakSlash:Embed(target)
    for _, name in pairs(mixins) do
        target[name] = OakSlash[name]
    end
    OakSlash.mixinTargets[target] = true
end
for target, _ in pairs(mixinTargets) do
    OakSlash:Embed(target)
end
--]]
