--[[
      Options menu for displays
]]

local ADDON_NAME, ns = ...

local L = ns.L

local Menu = {}

local Locked = {
    keepShownOnClick = 1,
    text = L["Locked"]
}
local Rotation = {
    text = L["Rotation"],
    keepShownOnClick = 1,
    hasArrow = true,
    notCheckable = true,
    menuList = {
        {
            text = L["Enabled"]
        },
        {
            text = L["Announce"]
        }
    }
}

local Share = {
    text = L["Share"],
    notCheckable = true
}

local Hide = {
    text = L["Hide"],
    notCheckable = true
}

local Delete = {
    text = L["Delete"],
    notCheckable = true
}

local Grow = {
    text = L["Orientation"],
    notCheckable = true,
    keepShownOnClick = 1,
    hasArrow = true,
    menuList = {
        {
            text = L["Up"],
        },
        {
            text = L["Down"],
        }
    }
}

local Refill = {
    text = L["Refill"],
    notCheckable = true
}

local AutoInclude = {
    keepShownOnClick = 1,
    text = L["Auto-fill"],
}

local Close = {
    text = L["Cancel"],
    notCheckable = true
}

local HotkeyReset = {
    keepShownOnClick = 1,
    text = L["Use reset bind"]
}


local SubsectionDivider = {
    text = "",
    isTitle = true,
    notCheckable = true,
    notClickable = true,
    iconOnly = true,
    icon = "Interface\\Common\\UI-TooltipDivider-Transparent",
    iconInfo = {
        tCoordLeft = 0,
        tCorrdRight = 1,
        tCoordTop = 0,
        tCoordBottom = 1,
        tSizeX = 0,
        tFitDropDownSizeX = true,
        tSizeY = 0
    }
}

local menuinfo = {
    {
        text = L["Actions"],
        isTitle = 1,
        notCheckable = true
    },
    Share,
    Refill,
    SubsectionDivider,
    {
        text = L["Options"],
        isTitle = 1,
        notCheckable = true
    },
    Rotation,
    HotkeyReset,
    Grow,
    AutoInclude,
    Locked,
    SubsectionDivider,
    {
        text = L["Other"],
        isTitle = 1,
        notCheckable = true
    },
    Hide,
    Delete,
    Close
}


local ddmenu = CreateFrame("Frame", "OakInterruptsDisplay_DrowDownMenu", UIParent, "UIDropDownMenuTemplate")

local function ToggleLock(disp)
    if disp.locked == true then
        ns.display.Unlock(disp)
    else
        ns.display.Lock(disp)
    end
end

local function ToggleRotation(disp)
    if disp.rotation then
        ns.display.SetRotation(disp, false)
    else
        ns.display.SetRotation(disp, true)
    end
end

local function ToggleAnnounce(disp)
    if not disp.rotation or not disp.rotation.announce then
        ns.display.SetRotation(disp, "ANNOUNCE")
    else
        ns.display.SetRotation(disp, true)
    end
end

local function ToggleAutoInclude(disp)
    ns.display.SetAutoInclude(disp, not disp.autoinclude)
end

local function ToggleHotkeyReset(disp)
    ns.display.SetHotkeyReset(disp, not disp.hotkey_reset)
end


local function Show(disp)
    Locked.checked = function() return disp.locked == true end
    Locked.func = function() ToggleLock(disp) end

    Rotation.menuList[1].checked = function() return disp.rotation ~= nil end
    Rotation.menuList[2].checked = function()
        return disp.rotation and disp.rotation.announce
    end
    Rotation.menuList[1].func = function() ToggleRotation(disp) end
    Rotation.menuList[2].func = function() ToggleAnnounce(disp) end

    HotkeyReset.checked = function() return disp.hotkey_reset == true end
    HotkeyReset.func = function() ToggleHotkeyReset(disp) end

    Share.disabled = not (disp.rotation and #disp.queue.frames > 0) and 1 or nil
    Share.func = function() ns.display.ShareRotation(disp) end

    Hide.func = function() ns.display.Hide(disp) end

    Delete.func = function() ns.display.Close(disp) end

    Grow.menuList[1].func = function() ns.display.SetDirection(disp, "up") end
    Grow.menuList[1].checked = function() return disp.queue.anchor == "BOTTOM" end
    Grow.menuList[2].func = function() ns.display.SetDirection(disp, "down") end
    Grow.menuList[2].checked = function() return not Grow.menuList[1].checked() end

    Refill.func = function() ns.display.Refill(disp) end

    AutoInclude.func = function() ToggleAutoInclude(disp) end
    AutoInclude.checked = disp.autoinclude

    EasyMenu(menuinfo, ddmenu, "cursor", 0, 0, nil, 1)
end

function Menu.Register(disp)
    -- create button
    local btn_opts = CreateFrame("Button", nil, disp.heading)

    btn_opts:SetHighlightTexture("Interface\\Buttons\\UI-OptionsButton")
    btn_opts:SetNormalTexture("Interface\\Buttons\\UI-OptionsButton")

    -- layout button
    local btnpad = 2
    local btnsize = disp.heading:GetHeight()
    btn_opts:SetSize(btnsize, btnsize)
    btn_opts:SetPoint("TOPRIGHT", disp.heading, "TOPRIGHT", -btnpad, 0)

    disp.btn_opts = btn_opts

    btn_opts:SetScript("OnClick", function(self, button, down)
        Show(disp)
    end)
end

--------------------------------------------------------------------------------
ns.Menu = Menu
