local ADDON_NAME, ns = ...

local L = ns.L
local Printer = LibStub:GetLibrary("OakPrint-1.0")(ADDON_NAME .. L[" Rearrange"])

local rearrange = {}

local GetCursorPosition = GetCursorPosition
--------------------------------------------------------------------------------

local function GetIndex(t, b)
    for k, v in ipairs(t) do
        if v == b then return k end
    end
end

local function MouseInFrame(frame, x, y, coord, s)
    local es = frame:GetEffectiveScale()
    x, y = x / es, y / es
    local h = frame:GetHeight()
    local w = frame:GetWidth()
    local ytest = y <= frame:GetTop() - h / s and y >= frame:GetBottom() + h / s
    local xtest = x <= frame:GetRight() and x >= frame:GetLeft()
    return (coord == "x" and xtest) or (coord == "y" and ytest) or (xtest and ytest)
end

function rearrange.BeginMoving(queue, frame)
    if not frame.isMoving then
        frame:SetAlpha(0.8)
        frame.isMoving = true
        local _, y = GetCursorPosition()
        queue.initPos = GetIndex(queue.frames, frame)
        queue.movingFrame = frame
        queue.unstick = true
    end
end

function rearrange.EndMoving(queue, frame)
    if frame.isMoving then
        frame:SetAlpha(1)
        frame.isMoving = false
        queue.initPos = nil
        queue.unstick = nil
        queue:AnchorFrames()
    end
end

local function GetNewIndex(queue, func, ...)
    for i, other in ipairs(queue.frames) do
        if i ~= queue.initPos and func(other, ...) then
            return i
        end
    end
    return queue.initPos
end

local function SwitchInterrupterDisplay(frame, from, to)
    Printer.debug(2, L["Switching %s to display %s"], frame.name, to.name)
    local name = frame.name
    from.watch_list[name] = false
    from.queue:RemoveFrame(frame)
    rearrange.EndMoving(from.queue, frame)
    from.queue:LayoutQueue()
    to.watch_list[name] = frame
    to.queue:AddFrame(frame)
    to.queue:ResizeContainer()
    rearrange.BeginMoving(to.queue, frame)
end

local function FramesOverlap(f1, f2)
    local es1, es2 = f1:GetEffectiveScale(), f2:GetEffectiveScale()
    local f1l, f1r, f1t, f1b = f1:GetLeft() * es1, f1:GetRight() * es1,
                                f1:GetTop() * es1, f1:GetBottom() * es1
    local f2l, f2r, f2t, f2b = f2:GetLeft() * es2, f2:GetRight() * es2,
                                f2:GetTop() * es2, f2:GetBottom() * es2
    return f1l < f2r and f2l < f1r and f1b < f2t and f2b < f1t
end

local function ChooseDisplay(cur_disp, frame)
    if FramesOverlap(cur_disp, frame) then return end
    for _, disp in pairs(ns.displays) do
        if not disp.watch_list[frame.name] then
            if FramesOverlap(disp, frame) then
                SwitchInterrupterDisplay(frame, cur_disp, disp)
                return disp
            end
        end
    end
end

function rearrange.MouseUpdate(disp, frame, elapsed)
    if frame.isMoving then
        local new_disp = ChooseDisplay(disp, frame)
        local queue = new_disp and new_disp.queue or disp.queue
        local x, y = GetCursorPosition()
        local s = frame:GetEffectiveScale()
        local ip = GetNewIndex(queue, MouseInFrame, x, y, "y", 3)
        queue:MoveFrame(frame, ip)
        queue.initPos = ip

        queue:AnchorFrames()
        frame:ClearAllPoints()
        frame:SetPoint("CENTER", nil, "BOTTOMLEFT", x / s, y / s)
        return new_disp
    end
end
--------------------------------------------------------------------------------
ns.rearrange = rearrange
