
local ADDON_NAME, ns = ...

local L = ns.L
local Printer = LibStub:GetLibrary("OakPrint-1.0")(ADDON_NAME .. L[" Rotation"])

local rotation = {}

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Rotation managment
--------------------------------------------------------------------------------

local function IndexInQueue(queue, val)
    for i, v in ipairs(queue.frames) do
        if v.name == val then
            return i
        end
    end
end

local function NextIndexIncremental(queue, next_index, data)
-- when inter-client comms are up, test HasFullControll() for each interrupter
    local off_cd_first = next_index
    local func = function(i)
        local name = queue.frames[i].name
        local t = data[name]
        if UnitIsDead(name) or not UnitIsConnected(name) then
            return
        end
        if t < GetTime() then
            return true, i
        elseif t < data[queue.frames[off_cd_first].name] then
            off_cd_first = i
        end
    end
    for i = next_index, #queue.frames do
        local r, ind = func(i)
        if r then return ind end
    end
    for i = 1, next_index do
        local r, ind = func(i)
        if r then return ind end
    end
    return off_cd_first
end

local function Announce(disporfmt, chan, ...)
    assert(disporfmt, "Usage: Announce(displayorfmt, chan, ...]). If chan is nil it will be automatically determined")
    local msg
    if type(disporfmt) == "string" then
        msg = format(disporfmt, ...)
    else
        local frames = disporfmt.queue.frames
        if #frames == 0 or not UnitAffectingCombat('player') then return end
        local name = frames and frames[disporfmt.rotation.next_index].name
        msg = format(L["Next interrupt is %s"], name)
    end

    local inraid = IsInRaid()
    local leader = UnitIsGroupLeader('player') or UnitIsGroupAssistant('player')

    if not chan then
        if UnitIsDead('player') then
            chan = inraid and "RAID" or "PARTY"
        else
            chan = inraid and leader and "RAID_WARNING" or "YELL"
        end
    end

    Printer.debug(2, L["Announcing '%s' to %s"], msg, chan)
    -- Do we need pcall? Can't remember why...
    pcall(SendChatMessage, msg, chan)
end

function rotation.AdvanceRotation(disp, data)
    if not disp.rotation then return end
    Printer.debug(2, L["Advancing rotation for %s"], disp.name)
    local cur_index = disp.rotation.next_index
    local queue = disp.queue
    local next_index = NextIndexIncremental(queue, cur_index, data)
    Printer.debug(3, L["%s - cur_index: %d, next_index: %d"], disp.name, cur_index, next_index)

    ns.display.RotationHighlight(disp)
    disp.rotation.next_index = next_index
    if disp.rotation.announce then
        Announce(disp)
    end
end

function rotation.AnnounceOrder(disp, nameflag)
    Printer.debug(2, L["Sharing order for %s"], disp.name)
    local order = {}
    local fmt = "%d. %s"
    local heading = L["Interrupt order"] .. (nameflag and " (%s)" or "")
    local chan = IsInRaid() and "RAID" or IsInGroup() and "PARTY" or "SAY"
    Announce(heading, chan, disp.name)
    for k, v in ipairs(disp.queue.frames) do
        Announce(fmt, chan, k, v.name)
    end
    if ns.Comms then ns.Comms.SendRotation(disp) end
end

function rotation.Reset(disp)
    if not disp.rotation then return end
    disp.rotation.next_index = 1
    if disp.rotation.announce then
        Announce(disp)
    end
end

ns.rotation = rotation
ns.rotation.Printer = Printer

-- TODO:
--       ideas for NextIndex:
--              a min cd remaining time so is doesn't skip people with 1/2/3 sec left on cd?
